/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { ActivatedRoute, Params, Router, UrlSegment } from '@angular/router';
import { AppService } from '@alfa-client/app-shared';
import { Mock, mock, useFromMock } from '@alfa-client/test-utils';
import { ResourceUri } from '@ngxp/rest';
import { OAuthService } from 'angular-oauth2-oidc';
import { of } from 'rxjs';
import { EMPTY_STRING } from '../../../tech-shared/src/lib/tech.util';
import { NavigationFacade } from './+state/navigation.facade';
import { NavigationService } from './navigation.service';


describe('NavigationService', () => {
	let service: NavigationService;

	let facade: Mock<NavigationFacade>;
	let appService: Mock<AppService>;
	let authService: Mock<OAuthService>;

	const router = {
		...mock(Router),
		url: EMPTY_STRING,
		events: of({})
	};

	const activatedRouteParams: Params = {};
	const activatedRoute = {
		...mock(ActivatedRoute),
		root: {
			firstChild: {
				params: of(activatedRouteParams)
			}
		},
		params: of(activatedRouteParams)
	};


	beforeEach(() => {
		facade = mock(NavigationFacade);
		appService = mock(AppService);

		service = new NavigationService(useFromMock(<any>router), useFromMock(<any>activatedRoute), useFromMock(facade), useFromMock(authService));
	});

	it('should be created', () => {
		expect(service).toBeTruthy();
	});

	describe('navigation', () => {

		describe('to vorganglist', () => {

			it('should navigate to all with empty url', () => {
				router.url = NavigationService.ROOT_PATH;

				service.navigateToVorgangList();

				expect(router.navigate).toHaveBeenCalledWith([NavigationService.ROOT_PATH, NavigationService.URL_PARAM_ALLE]);
			})

			it('should navigate to meine with url containing "meine"', () => {
				router.url = NavigationService.MEINE_VORGAENGE_PATH;;

				service.navigateToVorgangList();

				expect(router.navigate).toHaveBeenCalledWith([NavigationService.ROOT_PATH, NavigationService.URL_PARAM_MY_VORGAENGE]);
			})

			it('should navigate to all with url containing "alle"', () => {
				router.url = NavigationService.ALLE_VORGAENGE_PATH;

				service.navigateToVorgangList();

				expect(router.navigate).toHaveBeenCalledWith([NavigationService.ROOT_PATH, NavigationService.URL_PARAM_ALLE]);
			})

			it('should navigate to unassigned with url containing "unassigned"', () => {
				router.url = NavigationService.UNASSIGNED_VORGAENGE_PATH;

				service.navigateToVorgangList();

				expect(router.navigate).toHaveBeenCalledWith([NavigationService.ROOT_PATH, NavigationService.URL_PARAM_UNASSIGNED]);
			})
		})

		it('should navigate to vorgang with given url', () => {
			const linkRel: ResourceUri = 'dummyUrl';

			service.navigateToVorgang(linkRel);

			expect(router.navigate).toHaveBeenCalledWith([NavigationService.ROOT_PATH, NavigationService.URL_PARAM_MY_VORGANG, linkRel]);
		})

		it('should navigate with search string', () => {
			router.url = NavigationService.ROOT_PATH;
			const searchString: string = 'testSearchString';

			service.search(searchString);

			expect(router.navigate).toHaveBeenCalledWith([NavigationService.ROOT_PATH, NavigationService.URL_PARAM_ALLE, NavigationService.URL_PARAM_SEARCH, encodeURIComponent(searchString)]);
		})

		it('should navigate to route by path segments', () => {
			const pathSegments: string[] = ['meine', 'neu'];

			service.navigateByPathSegments(pathSegments);

			expect(router.navigate).toHaveBeenCalledWith(pathSegments)
		})
	})

	describe('isVorgangListPage', () => {

		it('should return true if params are empty', () => {
			const isPage: boolean = NavigationService.isVorgangListPage({ });

			expect(isPage).toBeTruthy();
		})

		it('return true if params contains status only', () => {
			const isPage: boolean = NavigationService.isVorgangListPage({ status: 'neu' });

			expect(isPage).toBeTruthy();
		})

		it('return true if params contains search only', () => {
			const isPage: boolean = NavigationService.isVorgangListPage({ search: 'SearchFor' });

			expect(isPage).toBeTruthy();
		})
	})

	describe('updateRouteData', () => {

		beforeEach(() => {
			service.updateRouteUrlSegment = jest.fn();
			service.updateRouteParameter = jest.fn();
		})

		it('should call updateRouteUrlSegment', () => {
			service.updateRouteData(<any>activatedRoute);

			expect(service.updateRouteUrlSegment).toHaveBeenCalled();
		})

		it('should call updateRouteParameter', () => {
			service.updateRouteData(<any>activatedRoute);

			expect(service.updateRouteParameter).toHaveBeenCalled();
		})
	})

	describe('updateRouteParameter', () => {

		it('should check if route changed', () => {
			service.isNewRoute = jest.fn();

			service.updateRouteParameter(<any>activatedRoute);

			expect(service.isNewRoute).toHaveBeenCalledWith(activatedRouteParams);
		})

		describe('on new route', () => {

			beforeEach(() => {
				service.isNewRoute = jest.fn();
				(<any>service).isNewRoute.mockReturnValue(true);
			})

			it('should update route parameter', () => {
				service.setRouteParameter = jest.fn();

				service.updateRouteParameter(<any>activatedRoute);

				expect(service.setRouteParameter).toHaveBeenCalledWith(activatedRouteParams);
			})
		})

		describe('on same route', () => {

			beforeEach(() => {
				service.isNewRoute = jest.fn();
				(<any>service).isNewRoute.mockReturnValue(false);
			})

			it('should NOT update route parameter if is new route', () => {
				service.setRouteParameter = jest.fn();

				service.updateRouteParameter(<any>activatedRoute);

				expect(service.setRouteParameter).not.toHaveBeenCalled();
			})
		})
	})

	describe('updateRouteUrlSegement', () => {

		it('should update routeUrlSegement by route url', () => {
			const urlSegement: UrlSegment[] = [<any>{ path: 'irgendein/pfad' }]
			const route = { ...mock(ActivatedRoute), url: of(urlSegement) };

			service.updateRouteUrlSegment(<any>route);

			expect(service.routeUrlSegment$.value).toEqual(urlSegement);
		})
	})

	describe('is postfach page', () => {

		it('should return true if "postfach" exists in url', () => {
			service.routeUrlSegment$.next(<any>[{ path: 'postfach' }]);

			const isPostfachPage: boolean = service.isPostfachPage();

			expect(isPostfachPage).toBeTruthy();
		})

		it('should return false if "postfach" not exists in url', () => {
			service.routeUrlSegment$.next(<any>[{ path: 'something else' }]);

			const isPostfachPage: boolean = service.isPostfachPage();

			expect(isPostfachPage).toBeFalsy();
		})

		it('should return false if routeUrl is empty', () => {
			service.routeUrlSegment$.next(<any>[]);

			const isPostfachPage: boolean = service.isPostfachPage();

			expect(isPostfachPage).toBeFalsy();
		})
	})

	describe('navigate', () => {

		const navigationRoute: string = 'dummyNavigationRoute';

		it('should call router with given navigationRoute', () => {
			service.navigate(navigationRoute);

			expect(router.navigate).toHaveBeenCalledWith([navigationRoute]);
		})
	})

	describe('getCurrentRouteData', () => {

		it('should call facade', () => {
			service.getCurrentRouteData();

			expect(facade.getCurrentRouteData).toHaveBeenCalled();
		})
	})

	describe('navigate by filter', () => {

		it('should navigate', () => {
			router.url = '/alle/neu';

			service.navigateByFilter('meine');

			expect(router.navigate).toHaveBeenCalledWith([EMPTY_STRING, 'meine', 'neu']);
		})
	})

	describe('getFilterPathSegment', () => {

		it('should return alle with empty url', () => {
			router.url = '/';

			const pathSegment: string = service.getFilterPathSegment();

			expect(pathSegment).toBe(NavigationService.URL_PARAM_ALLE);
		})

		it('should return alle with non-filter url', () => {
			router.url = '/vorgang/123';

			const pathSegment: string = service.getFilterPathSegment();

			expect(pathSegment).toBe(NavigationService.URL_PARAM_ALLE);
		})

		it('should return meine with url containing "meine"', () => {
			router.url = '/meine/neu';

			const pathSegment: string = service.getFilterPathSegment();

			expect(pathSegment).toBe(NavigationService.URL_PARAM_MY_VORGAENGE);
		})

		it('should return alle with url containing "alle', () => {
			router.url = '/alle/neu';

			const pathSegment: string = service.getFilterPathSegment();

			expect(pathSegment).toBe(NavigationService.URL_PARAM_ALLE);
		})

		it('should return unassigned with url containing "unassigned"', () => {
			router.url = '/unassigned/neu';

			const pathSegment: string = service.getFilterPathSegment();

			expect(pathSegment).toBe(NavigationService.URL_PARAM_UNASSIGNED);
		})
	})

	describe('isFilterPathSegment', () => {
		it('should return true for "alle"', () => {
			const result: boolean = service.isFilterPathSegment(NavigationService.URL_PARAM_ALLE);

			expect(result).toBeTruthy();
		})

		it('should return true for "meine"', () => {
			const result: boolean = service.isFilterPathSegment(NavigationService.URL_PARAM_MY_VORGAENGE);

			expect(result).toBeTruthy();
		})

		it('should return true for "unassigned"', () => {
			const result: boolean = service.isFilterPathSegment(NavigationService.URL_PARAM_UNASSIGNED);

			expect(result).toBeTruthy();
		})

		it('should return false for "vorgang"', () => {
			const result: boolean = service.isFilterPathSegment(NavigationService.URL_PARAM_MY_VORGANG);

			expect(result).toBeFalsy();
		})
	})

	describe('isNotRootPath', () => {
		it('should return true for non-root paths', () => {
			router.url = '/alle';

			const result: boolean = service.isNotRootPath();

			expect(result).toBeTruthy();
		})

		it('should return false for root path', () => {
			router.url = '/';

			const result: boolean = service.isNotRootPath();

			expect(result).toBeFalsy();
		})
	})
})