/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

import { formatDate, registerLocaleData } from '@angular/common';
import localeDe from '@angular/common/locales/de';
import { BinaryFileListResource, DownloadBinaryFileAsPdfAction } from '@alfa-client/binary-file-shared';
import { StateResource, createStateResource } from '@alfa-client/tech-shared';
import { Mock, mock, useFromMock } from '@alfa-client/test-utils';
import { VorgangHeaderLinkRel, VorgangWithEingangResource } from '@alfa-client/vorgang-shared';
import { Store } from '@ngrx/store';
import { getUrl } from '@ngxp/rest';
import { createBinaryFileListResource } from 'libs/binary-file-shared/test/binary-file';
import { createPostfachMailResource } from 'libs/postfach-shared/test/postfach';
import { createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { Subject, of } from 'rxjs';
import { PostfachMailResource } from '../postfach.model';
import { PostfachFacade } from './postfach.facade';

import * as BinaryFileActions from '@alfa-client/binary-file-shared';
import * as PostfachActions from './postfach.actions';
import * as PostfachSelectors from './postfach.selectors';

registerLocaleData(localeDe);

describe('PostfachFacade', () => {

	let facade: PostfachFacade;
	let store: Mock<Store>;

	let selectionSubject: Subject<any>;

	beforeEach(() => {
		store = mock(Store);

		selectionSubject = new Subject();

		store.select.mockReturnValue(selectionSubject);
		store.dispatch = jest.fn();

		facade = new PostfachFacade(useFromMock(<any>store));
	})

	it('is initialized', () => {
		expect(facade).toBeTruthy();
	})

	describe('startDownloadPdf', () => {

		it('should dispatch "startDownloadAsPdf" action', () => {
			facade.startDownloadPdf();

			expect(store.dispatch).toHaveBeenCalledWith(PostfachActions.startDownloadAsPdf());
		});
	})

	describe('isDownloadPdfInProgress', () => {

		it('should return value', (done) => {
			facade.isDownloadPdfInProgress().subscribe(isInProgress => {
				expect(isInProgress).toBeTruthy();
				done();
			});

			selectionSubject.next(true);
		});
	})

	describe('downloadPdf', () => {

		const vorgang: VorgangWithEingangResource = createVorgangWithEingangResource();
		const downloadAction: DownloadBinaryFileAsPdfAction = <any>{};

		beforeEach(() => {
			facade.createDownloadPdfProps = jest.fn().mockReturnValue(downloadAction);
		})

		it('should dispatch "downloadPdf" action', () => {
			facade.downloadPdf(vorgang);

			expect(store.dispatch).toHaveBeenCalledWith(BinaryFileActions.downloadPdf(downloadAction));
		})

		it('should create props', () => {
			facade.downloadPdf(vorgang);

			expect(facade.createDownloadPdfProps).toHaveBeenCalledWith(vorgang);
		})
	})

	describe('createDownloadPdfProps', () => {

		const vorgang: VorgangWithEingangResource = createVorgangWithEingangResource([VorgangHeaderLinkRel.POSTFACH_MAILS]);

		it('should have uri', () => {
			const props: DownloadBinaryFileAsPdfAction = facade.createDownloadPdfProps(vorgang);

			expect(props.uri).toBe(getUrl(vorgang, VorgangHeaderLinkRel.POSTFACH_MAILS));
		})

		it('should have fileName', () => {
			const props: DownloadBinaryFileAsPdfAction = facade.createDownloadPdfProps(vorgang);

			expect(props.fileName).toBe(`${vorgang.nummer}_${formatDate(new Date(), 'yyyyMMdd', 'de')}_Nachrichten.pdf`);
		})

		it('should have "downloadAsPdfSuccess" as success action', () => {
			const props: DownloadBinaryFileAsPdfAction = facade.createDownloadPdfProps(vorgang);

			expect(props.successAction).toBe(PostfachActions.downloadAsPdfSuccess);
		})

		it('should have "downloadAsPdfFailed" as failure action', () => {
			const props: DownloadBinaryFileAsPdfAction = facade.createDownloadPdfProps(vorgang);

			expect(props.failureAction).toBe(facade.createFailureAction);
		})
	})

	describe('getAttachments', () => {

		const attachmentListStateResource: StateResource<BinaryFileListResource> = createStateResource(createBinaryFileListResource());

		it('should select attachmentList', () => {
			facade.getAttachmentList();

			expect(store.select).toHaveBeenCalledWith(PostfachSelectors.attachmentList);
		})

		it('should return selected value', (done) => {
			store.select.mockReturnValue(of(attachmentListStateResource));

			facade.getAttachmentList().subscribe(attachmentList => {
				expect(attachmentList).toBe(attachmentListStateResource);
				done();
			});
		})
	})

	describe('loadAttachmentList', () => {

		const postfachNachricht: PostfachMailResource = createPostfachMailResource();

		it('should dispatch action', () => {
			facade.loadAttachmentList(postfachNachricht);

			expect(store.dispatch).toHaveBeenCalledWith(PostfachActions.loadAttachmentList({ postfachNachricht }));
		})
	})

	describe('clearAttachmentList', () => {

		it('should dispatch action', () => {
			facade.clearAttachmentList();

			expect(store.dispatch).toHaveBeenCalledWith(PostfachActions.clearAttachmentList());
		})
	})
});
