/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { UntypedFormBuilder, UntypedFormGroup } from '@angular/forms';
import { ApiError, hasError, setValidationError, StateResource } from '@alfa-client/tech-shared';
import { Resource } from '@ngxp/rest';
import { isNil } from 'lodash-es';
import { Observable } from 'rxjs';
import { map } from 'rxjs/operators';

export abstract class AbstractFormService {
	form: UntypedFormGroup;
	pathPrefix: string;
	source: any;

	constructor(public formBuilder: UntypedFormBuilder) {
		this.form = this.initForm();
	}

	protected abstract initForm(): UntypedFormGroup;

	public submit(): Observable<StateResource<Resource>> {
		return this.doSubmit().pipe(map(result => this.handleResponse(result)));
	}

	protected abstract doSubmit(): Observable<StateResource<Resource>>;

	handleResponse(result: StateResource<Resource>): StateResource<Resource> {
		if (result.loading) return result;
		if (hasError(result)) {
			this.setError(result.error);
		}
		return result;
	}

	patch(valueToPatch: any): void {
		this.form.reset();
		this.form.patchValue(valueToPatch);

		this.source = valueToPatch;

		this.doAfterPatch(valueToPatch);
	}

	doAfterPatch(source: any): void {
		//No Implementation here for abstract class
	}

	setError(apiError: ApiError): void {
		apiError.issues.forEach(issue => setValidationError(this.form, issue, this.getPathPrefix()))
	}

	protected abstract getPathPrefix(): string;

	getFormValue(): any {
		return this.form.value;
	}

	protected getSourceValue(): any {
		return this.source;
	}

	public isPatch(): boolean {
		return !isNil(this.source);
	}
}