/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { HttpErrorResponse, HttpRequest, HttpStatusCode } from '@angular/common/http';
import { TestBed } from '@angular/core/testing';
import { MatDialogModule } from '@angular/material/dialog';
import { AuthService } from '@alfa-client/app-shared';
import { HttpErrorHandler } from '@alfa-client/tech-shared';
import { Mock, mock } from '@alfa-client/test-utils';
import { createApiError } from 'libs/tech-shared/test/error';
import { Subject } from 'rxjs';
import { SnackBarService } from '../snackbar/snackbar.service';
import { DialogService } from '../ui/dialog/dialog.service';
import { Messages } from '../ui/messages';
import { HttpErrorInterceptor } from './http-error.interceptor';

describe('HttpErrorInterceptor', () => {
	let interceptor: HttpErrorInterceptor;

	const dialogService: Mock<DialogService> = mock(DialogService);
	const snackbarService: Mock<SnackBarService> = mock(SnackBarService);
	const httpErrorHandler: Mock<HttpErrorHandler> = mock(HttpErrorHandler);
	const authService: Mock<AuthService> = mock(AuthService);

	beforeEach(() => TestBed.configureTestingModule({
		imports: [
			MatDialogModule
		],
		providers: [
			HttpErrorInterceptor,
			{
				provide: DialogService,
				useValue: dialogService
			},
			{
				provide: SnackBarService,
				useValue: snackbarService
			},
			{
				provide: HttpErrorHandler,
				useValue: httpErrorHandler
			},
			{
				provide: AuthService,
				useValue: authService
			}
		]
	}));

	beforeEach(() => {
		interceptor = TestBed.inject(HttpErrorInterceptor);
	})

	it('should be created', () => {
		expect(interceptor).toBeTruthy();
	});

	describe('intercept', () => {

		const response = new HttpErrorResponse({});
		const error = {response};
		const subject: Subject<any> = new Subject();
		const httpHandler = { handle: () => subject };
		const request: HttpRequest<unknown> = new HttpRequest('GET', '/test');

		it('should call http error handler', () => {
			interceptor.handleErrorResponse = jest.fn();

			interceptor.intercept(request, <any>httpHandler).subscribe();
			subject.error(error);

			expect(httpErrorHandler.shouldDoDefaultHandling).toHaveBeenCalled();
		})

		it('should call handleErrorResponse with error', () => {
			httpErrorHandler.shouldDoDefaultHandling.mockReturnValue(true);
			interceptor.handleErrorResponse = jest.fn();

			interceptor.intercept(request, <any>httpHandler).subscribe();
			subject.error(error);

			expect(interceptor.handleErrorResponse).toHaveBeenCalledWith({response}, true);
		})
	})

	describe('handleErrorResponse', () => {

		const response: HttpErrorResponse = new HttpErrorResponse({ error: createApiError(), status: 500 });
		const error = {response};

		beforeEach(() => {
			interceptor.handleError = jest.fn();
		})

		it('should call handleError if defaultHandling is enabled', () => {
			interceptor.handleErrorResponse(error, true);

			expect(interceptor.handleError).toHaveBeenCalledWith(error);
		})

		it('should not call handleError if defaultHandling is disabled', () => {
			interceptor.handleErrorResponse(error, false);

			expect(interceptor.handleError).not.toHaveBeenCalled();
		})
	})

	describe('handleError', () => {

		it('should open dialog on server error', () => {
			const response: HttpErrorResponse = new HttpErrorResponse({ error: createApiError(), status: 500 });
			const error = {response};
			interceptor.handleError(error);

			expect(dialogService.openApiErrorInfo).toHaveBeenCalledWith(response.error);
		})

		it('should logout on error 401', () => {
			const response: HttpErrorResponse = new HttpErrorResponse({ error: createApiError(), status: HttpStatusCode.Unauthorized });
			const error = {response};

			interceptor.handleError(error);

			expect(authService.logoutWithConfirmation).toHaveBeenCalled();
		})

		it('should call handleForbiddenError on error 403', () => {
			interceptor.handleForbiddenError = jest.fn();
			const response: HttpErrorResponse = new HttpErrorResponse({ status: HttpStatusCode.Forbidden });
			const error = {response};

			interceptor.handleError(error);

			expect(interceptor.handleForbiddenError).toHaveBeenCalled();
		})

		describe('for forbidden, ',() => {

			it('should NOT call snackbarService if already visible', () => {
				snackbarService.isVisible.mockReturnValue(true);

				interceptor.handleForbiddenError();

				expect(snackbarService.showError).not.toHaveBeenCalled();
			})

			it('should call snackbarService if not visible', () => {
				snackbarService.isVisible.mockReturnValue(false);

				interceptor.handleForbiddenError();

				expect(snackbarService.showError).toHaveBeenCalledWith(Messages.HTTP_STATUS_FORBIDDEN);
			})
		})
	})

	describe('handleUnauthorizedError', () => {
		it('should call authService.logOut', () => {
			interceptor.handleUnauthorizedError();

			expect(authService.logoutWithConfirmation).toHaveBeenCalled();
		})
	})
});