/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { TestBed } from '@angular/core/testing';
import { MatSnackBar } from '@angular/material/snack-bar';
import { MatSnackBarConfig, MatSnackBarRef } from '@angular/material/snack-bar';
import { faker } from '@faker-js/faker';
import { CommandResource, CommandStatus } from '@alfa-client/command-shared';
import { mock, useFromMock } from '@alfa-client/test-utils';
import { createCommandResource } from 'libs/command-shared/test/command';
import { SnackbarErrorComponent } from './snackbar-error/snackbar-error.component';
import { SnackbarInfoComponent } from './snackbar-info/snackbar-info.component';
import { SnackBarService } from './snackbar.service';

describe('SnackBarService', () => {
	let service: SnackBarService;
	let matSnackBar: MatSnackBar;

	const mockMatSnackBar = {
		openFromComponent: jest.fn()
	};

	const commandResource: CommandResource = createCommandResource();
	const message: string = faker.lorem.words(5);

	beforeEach(async () => {
		TestBed.configureTestingModule({
			providers: [
				{ provide: MatSnackBar, useValue: mockMatSnackBar },
			],
		}).compileComponents();
		service = TestBed.inject(SnackBarService);
		matSnackBar = TestBed.inject(MatSnackBar);
	});

	it('should create', () => {
		expect(service).toBeDefined();
	})

	describe('show', () => {

		beforeEach(() => {
			service.listenOnRevokeAction = jest.fn();
			service.listenOnAfterDismissed = jest.fn();
			service.closeSnackBarIfVisible = jest.fn();
		})

		it('should close snackbar if exists', () => {
			service.show(commandResource, message);

			expect(service.closeSnackBarIfVisible).toHaveBeenCalled();
		})

		it('should open from component', () => {
			service.show(commandResource, message);

			expect(matSnackBar.openFromComponent).toHaveBeenCalledWith(SnackbarInfoComponent, {
				data: { message, commandResource }, duration: 10000
			});
		})

		it('should listen to snackbar revoke action', () => {
			service.show(commandResource, message, null);

			expect(service.listenOnRevokeAction).toHaveBeenCalledWith(null);
		})

		it('should listen to snackbar after dismissed', () => {
			service.show(commandResource, message, null);

			expect(service.listenOnAfterDismissed).toHaveBeenCalled();
		})

		it('should open error snackbar when status is ERROR', () => {
			const commandWithError = { ...commandResource, status: CommandStatus.ERROR }
			service.show(commandWithError, message, null);

			expect(matSnackBar.openFromComponent).toHaveBeenCalledWith(SnackbarErrorComponent, {
				data: { message }, duration: 10000, panelClass: 'error'
			});
		});
	})

	describe('show error', () => {

		beforeEach(() => {
			service.listenOnAfterDismissed = jest.fn();
			service.closeSnackBarIfVisible = jest.fn();
		})

		it('should close snackbar if exists', () => {
			service.showError(message);

			expect(service.closeSnackBarIfVisible).toHaveBeenCalled();
		})


		it('should open from component', () => {
			service.showError(message);

			expect(matSnackBar.openFromComponent).toHaveBeenCalledWith(SnackbarErrorComponent, {
				data: { message }, duration: 10000, panelClass: 'error'
			});
		})

		it('should listen to snackbar after dismissed', () => {
			service.showError(message);

			expect(service.listenOnAfterDismissed).toHaveBeenCalled();
		})
	})

	describe('show info', () => {

		beforeEach(() => {
			service.listenOnAfterDismissed = jest.fn();
			service.closeSnackBarIfVisible = jest.fn();
		})

		it('should close snackbar if exists', () => {
			service.showInfo(message);

			expect(service.closeSnackBarIfVisible).toHaveBeenCalled();
		})

		it('should open from component', () => {
			service.showInfo(message);

			const config: MatSnackBarConfig = { data: { message }, duration: 10000 };
			expect(matSnackBar.openFromComponent).toHaveBeenCalledWith(SnackbarInfoComponent, config);
		})

		it('should listen to snackbar after dismissed', () => {
			service.showInfo(message);

			expect(service.listenOnAfterDismissed).toHaveBeenCalled();
		})
	})

	describe('close snackbar if visible', () => {

		beforeEach(() => {
			service.closeSnackBar = jest.fn();
		})

		it('should dismiss snack bar if ref exists ', () => {
			service.snackBarRef = <any>{};

			service.closeSnackBarIfVisible();

			expect(service.closeSnackBar).toHaveBeenCalled();
		})

		it('should do nothing if ref not exists ', () => {
			service.snackBarRef = undefined;

			service.closeSnackBarIfVisible();

			expect(service.closeSnackBar).not.toHaveBeenCalled();
		})
	})

	describe('isVisible', () => {

		it('should return true if snackBarRef is not undefined', () => {
			service.snackBarRef = undefined;

			const isVisible: boolean = service.isVisible();

			expect(isVisible).toBeFalsy();
		})

		it('should return false if snackBarRef is defined', () => {
			service.snackBarRef = useFromMock(mock(MatSnackBarRef<SnackbarInfoComponent>));

			const isVisible: boolean = service.isVisible();

			expect(isVisible).toBeTruthy();
		})
	})

	describe('isNotVisible', () => {

		it('should return false if snackBarRef is not undefined', () => {
			service.snackBarRef = undefined;

			const isVisible: boolean = service.isNotVisible();

			expect(isVisible).toBeTruthy();
		})

		it('should return true if snackBarRef is defined', () => {
			service.snackBarRef = useFromMock(mock(MatSnackBarRef<SnackbarInfoComponent>));

			const isVisible: boolean = service.isNotVisible();

			expect(isVisible).toBeFalsy();
		})
	})
})
