/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { HttpErrorResponse, HttpStatusCode } from '@angular/common/http';
import { ApiRootFacade, ApiRootLinkRel, ApiRootResource } from '@alfa-client/api-root-shared';
import { NavigationService } from '@alfa-client/navigation-shared';
import { ApiError, MessageCode, StateResource, createEmptyStateResource, createErrorStateResource, createStateResource } from '@alfa-client/tech-shared';
import { Mock, mock, useFromMock } from '@alfa-client/test-utils';
import { VorgangHeaderLinkRel, VorgangResource, VorgangService } from '@alfa-client/vorgang-shared';
import { getLink } from '@ngxp/rest';
import { cold, hot } from 'jest-marbles';
import { createApiRootResource } from 'libs/api-root-shared/test/api-root';
import { createApiError } from 'libs/tech-shared/test/error';
import { createHttpErrorResponse } from 'libs/tech-shared/test/http';
import { createUserProfileListResource, createUserProfileResource } from 'libs/user-profile-shared/test/user-profile';
import { vorgangWithEingang } from 'libs/vorgang-shared/src/lib/+state/vorgang.selectors';
import { createVorgangResource } from 'libs/vorgang-shared/test/vorgang';
import { of, throwError } from 'rxjs';
import { UserProfileFacade } from './+state/user-profile.facade';
import { UserProfileListResource, UserProfileResource } from './user-profile.model';
import { UserProfileRepository } from './user-profile.repository';
import { UserProfileService } from './user-profile.service';

describe('UserProfileService', () => {
	let service: UserProfileService;
	let repository: Mock<UserProfileRepository>;
	let apiRootFacade: Mock<ApiRootFacade>;
	let navigationService: Mock<NavigationService>;
	let vorgangService: Mock<VorgangService>;
	let facade: Mock<UserProfileFacade>;

	const vorgang: VorgangResource = createVorgangResource([VorgangHeaderLinkRel.ASSIGNED_TO]);
	const userProfile: UserProfileResource = createUserProfileResource();
	const userProfileList: UserProfileListResource = createUserProfileListResource();

	beforeEach(() => {
		repository = mock(UserProfileRepository);
		apiRootFacade = mock(ApiRootFacade);
		vorgangService = mock(VorgangService);
		navigationService = mock(NavigationService);
		navigationService.urlChanged = jest.fn();
		navigationService.urlChanged.mockReturnValue(of({}));
		facade = mock(UserProfileFacade);

		service = new UserProfileService(useFromMock(repository), useFromMock(apiRootFacade), useFromMock(navigationService), useFromMock(vorgangService), useFromMock(facade));
	})

	it('should created', () => {
		expect(service).toBeTruthy();
	})

	describe('getCurrentUser', () => {

		const apiRootResource: StateResource<ApiRootResource> = createStateResource(createApiRootResource());
		const currentUserProfile: StateResource<UserProfileResource> = createStateResource(createUserProfileResource());

		it('should call get current user', () => {
			service.getCurrentUserFromState = jest.fn();

			service.getCurrentUser();

			expect(service.getCurrentUserFromState).toHaveBeenCalled();
		})

		it('should call apiRoot facade', () => {
			service.getCurrentUser();

			expect(apiRootFacade.getApiRoot).toHaveBeenCalled();
		})

		it('should return initial values', () => {
			service.getCurrentUserFromState = jest.fn().mockReturnValue(hot('-a', { a: currentUserProfile }));
			apiRootFacade.getApiRoot.mockReturnValue(hot('-a', { a: apiRootResource }));

			const result = service.getCurrentUser();

			expect(result).toBeObservable(cold('ab', { a: createEmptyStateResource(true), b: currentUserProfile }));
		})

		describe('handle get current user', () => {

			beforeEach(() => {
				service.handleGetCurrentUser = jest.fn();
			})

			it('should be called if apiRoot exists', () => {
				const apiRootResource: StateResource<ApiRootResource> = createStateResource(createApiRootResource([ApiRootLinkRel.CURRENT_USER]));
				apiRootFacade.getApiRoot.mockReturnValue(of(apiRootResource));

				service.getCurrentUser().subscribe();

				expect(service.handleGetCurrentUser).toHaveBeenCalled();
			})

			it('should NOT be called if apiRoot is null', () => {
				apiRootFacade.getApiRoot.mockReturnValue(of(createEmptyStateResource()));

				service.getCurrentUser().subscribe();

				expect(service.handleGetCurrentUser).not.toHaveBeenCalled();
			})
		})

		describe('handleGetCurrentUser', () => {

			beforeEach(() => {
				service.loadCurrentUser = jest.fn();
			})

			describe('with existing link', () => {

				const apiRoot: ApiRootResource = createApiRootResource([ApiRootLinkRel.CURRENT_USER]);

				it('should load current userProfile if link exists and no loading required', () => {
					service.handleGetCurrentUser(createEmptyStateResource(), apiRoot);

					expect(service.loadCurrentUser).toHaveBeenCalledWith(apiRoot);
				})

				it('should not load current userProfile on error', () => {
					service.handleGetCurrentUser(createErrorStateResource(createApiError()), apiRoot);

					expect(service.loadCurrentUser).not.toHaveBeenCalled();
				})
			})

			describe('on missing link', () => {

				const apiRoot: ApiRootResource = createApiRootResource();

				it('should not load current userProfile if link NOT exists', () => {
					service.handleGetCurrentUser(currentUserProfile, apiRoot);

					expect(service.loadCurrentUser).not.toHaveBeenCalled();
				})

				it('should not load current userProfile if already loaded', () => {
					service.handleGetCurrentUser(currentUserProfile, apiRoot);

					expect(service.loadCurrentUser).not.toHaveBeenCalled();
				})

				it('should not load current userProfile if is loading', () => {
					service.handleGetCurrentUser({ ...createEmptyStateResource(), loading: true }, apiRoot);

					expect(service.loadCurrentUser).not.toHaveBeenCalled();
				})
			})
		})

		describe('load current user profile', () => {

			const apiRoot: ApiRootResource = createApiRootResource();
			const userProfile: UserProfileResource = createUserProfileResource();

			beforeEach(() => {
				repository.getCurrentUser.mockReturnValue(of(userProfile));
			})

			it('should set stateResource loading true', () => {
				service.setCurrentUserLoading = jest.fn();

				service.loadCurrentUser(apiRoot);

				expect(service.setCurrentUserLoading).toHaveBeenCalled();
			})

			it('should call repository', () => {
				service.setCurrentUserLoading = jest.fn();

				service.loadCurrentUser(apiRoot);

				expect(repository.getCurrentUser).toHaveBeenCalledWith(apiRoot);
			})

			it('should createStateResource', () => {
				service.createStateResourceForUpdate = jest.fn();

				service.loadCurrentUser(apiRoot);

				expect(service.createStateResourceForUpdate).toHaveBeenCalledWith(userProfile);
			})

			it('should set value', () => {
				service.setCurrentUser = jest.fn();

				service.loadCurrentUser(apiRoot);

				expect(service.setCurrentUser).toHaveBeenCalled();
			})

			describe('with error', () => {

				const errorResponse: HttpErrorResponse = createHttpErrorResponse();

				it('should call error handler', () => {
					repository.getCurrentUser.mockReturnValue(throwError(() => errorResponse));
					service.handleError = jest.fn();

					service.loadCurrentUser(apiRoot);

					expect(service.handleError).toHaveBeenCalledWith(errorResponse.error);
				});
			});
		})
	})

	describe('load user profile', () => {

		const linkRel: string = VorgangHeaderLinkRel.ASSIGNED_TO;

		beforeEach(() => {
			repository.getUserProfile.mockReturnValue(of(userProfile));
			service.createIfNotExist(getLink(vorgang, linkRel).href);
			service.updateUserProfile = jest.fn();
		})

		it('should call repository', () => {
			service.loadUserProfile(vorgang, VorgangHeaderLinkRel.ASSIGNED_TO);

			expect(repository.getUserProfile).toHaveBeenCalledWith(vorgang, linkRel);
		})

		it('should update value', () => {
			service.createStateResourceForUpdate = jest.fn();
			(<any>service.createStateResourceForUpdate).mockReturnValue(createStateResource(userProfile));

			service.loadUserProfile(vorgang, VorgangHeaderLinkRel.ASSIGNED_TO);

			expect(service.updateUserProfile).toHaveBeenCalledWith(vorgang, linkRel, createStateResource(userProfile));
		})

		describe('with error', () => {

			const errorResponse: HttpErrorResponse = createHttpErrorResponse();

			it('should call error handler', () => {
				repository.getUserProfile.mockReturnValue(throwError(() => errorResponse));
				service.handleError = jest.fn();

				service.loadUserProfile(vorgang, linkRel);

				expect(service.handleError).toHaveBeenCalledWith(errorResponse.error);
			});

			it('should respond with ErrorStateResource for 404 errors', () => {
				const result: ApiError = service.handleErrorByStatus({ ...errorResponse, status: HttpStatusCode.NotFound });

				expect(result).toEqual(errorResponse.error);
			});

			it('should respond with ErrorStateResource for 503 errors', () => {
				const result: ApiError = service.handleErrorByStatus({ ...errorResponse, status: HttpStatusCode.ServiceUnavailable });

				expect(result).toEqual(errorResponse.error);
			});

			it('should respond with ErrorStateResource for 503 errors', () => {
				const result: ApiError = service.handleErrorByStatus({ ...errorResponse, status: 0 });

				expect(result.issues[0].messageCode).toEqual(MessageCode.SERVICE_UNAVAILABLE);
			});
		});
	})

	describe('createStateResourceForUpdate', () => {

		beforeEach(() => {
			service.updateUserProfile = jest.fn();
		})

		it('should return stateResource with userProfile', () => {
			const stateResource: StateResource<UserProfileResource> = service.createStateResourceForUpdate(userProfile);

			expect(stateResource).toEqual(createStateResource(userProfile));
		})

		it('should return stateResource with userProfile', () => {
			const userProfile: ApiError = createApiError();

			const errorStateResource: StateResource<UserProfileResource> = service.createStateResourceForUpdate(userProfile);

			expect(errorStateResource).toEqual(createErrorStateResource(userProfile));
		})
	})

	describe('search', () => {

		const searchBy: string = 'SearchMe';

		beforeEach(() => {
			repository.search.mockReturnValue(hot('-a', { a: userProfileList }))
			vorgangService.getVorgangWithEingang.mockReturnValue(of(createStateResource(vorgangWithEingang)));
		})

		it('should call repository', () => {
			service.search(searchBy);

			expect(repository.search).toHaveBeenCalledWith(vorgangWithEingang, searchBy);
		})

		it('should call vorgang service', () => {
			service.search(searchBy);

			expect(vorgangService.getVorgangWithEingang).toHaveBeenCalled();
		})
	})

	describe('getUserProfileSearchVisibility', () => {

		it('should call facade', () => {
			service.getUserProfileSearchVisibility();

			expect(facade.getUserProfileSearchVisibility).toHaveBeenCalled();
		})
	})

	describe('showUserProfileSearch', () => {

		it('should call facade', () => {
			service.showUserProfileSearch();

			expect(facade.showUserProfileSearch).toHaveBeenCalled();
		})
	})

	describe('hideUserProfileSearch', () => {

		it('should call facade', () => {
			service.hideUserProfileSearch();

			expect(facade.hideUserProfileSearch).toHaveBeenCalled();
		})
	})
})