/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { HttpErrorResponse, HttpStatusCode } from '@angular/common/http';
import { TestBed } from '@angular/core/testing';
import { ApiRootFacade, ApiRootLinkRel, ApiRootResource } from '@alfa-client/api-root-shared';
import { BinaryFileListResource, LoadBinaryFileListProps, loadBinaryFileList } from '@alfa-client/binary-file-shared';
import { CommandListResource, LoadCommandListProps, loadCommandList } from '@alfa-client/command-shared';
import { ApiError, BlobWithFileName, createStateResource } from '@alfa-client/tech-shared';
import { mock } from '@alfa-client/test-utils';
import { SnackBarService } from '@alfa-client/ui';
import { provideMockActions } from '@ngrx/effects/testing';
import { Action } from '@ngrx/store';
import { TypedAction } from '@ngrx/store/src/models';
import { MockStore, provideMockStore } from '@ngrx/store/testing';
import { ResourceUri } from '@ngxp/rest';
import { cold, hot } from 'jest-marbles';
import { ColdObservable } from 'jest-marbles/typings/src/rxjs/cold-observable';
import { createApiRootResource } from 'libs/api-root-shared/test/api-root';
import { createBinaryFileListResource } from 'libs/binary-file-shared/test/binary-file';
import { createCommandListResource } from 'libs/command-shared/test/command';
import { createApiError, createHttpErrorResponse } from 'libs/tech-shared/test/error';
import { createVorgangListResource, createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { Observable, of } from 'rxjs';
import { DUMMY_ACTION } from '../../../../tech-shared/test/ngrx';
import { VorgangWithEingangLinkRel } from '../vorgang.linkrel';
import { AdditionalActions, VorgangFilter, VorgangListResource, VorgangWithEingangResource } from '../vorgang.model';
import { VorgangEffects } from './vorgang.effects';
import { VorgangFacade } from './vorgang.facade';
import { VorgangRepository } from './vorgang.repository';

import faker from '@faker-js/faker';

import * as BinaryFileActions from '../../../../binary-file-shared/src/lib/+state/binary-file.actions';
import * as VorgangActions from './vorgang.actions';
import * as VorgangSelectors from './vorgang.selectors';

describe('VorgangEffects', () => {
	let actions: Observable<Action>;
	let effects: VorgangEffects;
	let store: MockStore;

	const apiRootFacade = mock(ApiRootFacade);
	const vorgangRepository = mock(VorgangRepository);
	const vorgangFacade = mock(VorgangFacade);
	const snackbarService = mock(SnackBarService);

	const vorgangList: VorgangListResource = createVorgangListResource();
	const vorgangWithEingang: VorgangWithEingangResource = createVorgangWithEingangResource();

	beforeEach(() => {
		TestBed.configureTestingModule({
			providers: [
				VorgangEffects,
				provideMockActions(() => actions),
				provideMockStore(),
				{
					provide: ApiRootFacade,
					useValue: apiRootFacade
				},
				{
					provide: VorgangRepository,
					useValue: vorgangRepository
				},
				{
					provide: VorgangFacade,
					useValue: vorgangFacade
				},
				{
					provide: SnackBarService,
					useValue: snackbarService
				}
			]
		});

		effects = TestBed.inject(VorgangEffects);

		store = TestBed.inject(MockStore);
		store.overrideSelector(VorgangSelectors.vorgangList, createStateResource(vorgangList));
	});

	describe('loadVorgangList', () => {

		const vorgangList: VorgangListResource = createVorgangListResource();
		const apiRoot: ApiRootResource = createApiRootResource();
		const linkRel: string = 'dummyLinkRel';
		const action = VorgangActions.loadVorgangList({ apiRoot, linkRel });

		it('should call repository', () => {
			actions = of(action);

			effects.loadVorgangList$.subscribe();

			expect(vorgangRepository.loadVorgangList).toHaveBeenCalledWith(apiRoot, linkRel);
		})

		it('should dispatch loadVorgangListSuccess action', () => {
			vorgangRepository.loadVorgangList.mockReturnValue(of(vorgangList));

			actions = hot('-a-|', { a: action });

			const expected = hot('-a-|', { a: VorgangActions.loadVorgangListSuccess({ vorgangList }) });
			expect(effects.loadVorgangList$).toBeObservable(expected);
		})

		it('should dispatch loadVorgangListFailure action', () => {
			const apiError: ApiError = createApiError()
			const error = { error: { error: apiError } };
			const errorResponse = cold('-#', {}, error);
			vorgangRepository.loadVorgangList = jest.fn(() => errorResponse);

			const expected = cold('--b', { b: VorgangActions.loadVorgangListFailure({ apiError }) });
			actions = hot('-a', { a: action });

			expect(effects.loadVorgangList$).toBeObservable(expected);
		})
	})

	describe('searchVorgaengeBy', () => {

		const apiRoot: ApiRootResource = createApiRootResource();
		const searchString: string = 'search like me';
		const linkRel: string = 'linkRelationName';
		const action = VorgangActions.searchVorgaengeBy({ apiRoot, searchString, linkRel });

		it('should call repository', () => {
			actions = of(action);

			effects.searchVorgaengeBy$.subscribe();

			expect(vorgangRepository.searchVorgaengeBy).toHaveBeenCalledWith(apiRoot, searchString, linkRel);
		})

		it('should dispatch searchVorgaengeBySuccess action', () => {
			vorgangRepository.searchVorgaengeBy.mockReturnValue(of(vorgangList));

			actions = hot('-a-|', { a: action });

			const expected = hot('-a-|', { a: VorgangActions.searchVorgaengeBySuccess({ vorgangList }) });
			expect(effects.searchVorgaengeBy$).toBeObservable(expected);
		})

		it('should dispatch searchVorgaengeByFailure action', () => {
			const error: HttpErrorResponse = createHttpErrorResponse();
			const errorResponse = cold('-#', {}, error);
			vorgangRepository.searchVorgaengeBy = jest.fn(() => errorResponse);

			const expected = cold('--b', { b: VorgangActions.searchVorgaengeByFailure({ httpErrorResponse: error }) });
			actions = hot('-a', { a: action });

			expect(effects.searchVorgaengeBy$).toBeObservable(expected);
		})
	})

	describe('loadNextPage', () => {

		const action = VorgangActions.loadNextPage();

		it('should select vorgangList from store', () => {
			store.select = jest.fn();
			actions = of(action);

			effects.loadNextPage$.subscribe();

			expect(store.select).toHaveBeenCalledWith(VorgangSelectors.vorgangList);
		})

		it('should call vorgang repository', () => {
			actions = of(action);

			effects.loadNextPage$.subscribe();

			expect(vorgangRepository.getNextVorgangListPage).toHaveBeenCalledWith(vorgangList);
		})

		it('should dispatch loadNextPageSuccess action', () => {
			vorgangRepository.getNextVorgangListPage.mockReturnValue(of(vorgangList));

			actions = hot('-a-|', { a: action });

			const expected = hot('-a-|', { a: VorgangActions.loadNextPageSuccess({ vorgangList }) });
			expect(effects.loadNextPage$).toBeObservable(expected);
		})

		it('should dispatch loadVorgangListFailure action', () => {
			const apiError: ApiError = createApiError()
			const error = { error: { error: apiError } };
			const errorResponse = cold('-#', {}, error);
			vorgangRepository.getNextVorgangListPage = jest.fn(() => errorResponse);

			const expected = cold('--c', { c: VorgangActions.loadVorgangListFailure({ apiError }) });
			actions = hot('-a', { a: action });

			expect(effects.loadNextPage$).toBeObservable(expected);
		})
	})

	describe('searchForPreview', () => {

		const vorgangList: VorgangListResource = createVorgangListResource();
		const apiRoot: ApiRootResource = createApiRootResource();
		const searchString: string = 'searchThisForMe';
		const linkRel: string = ApiRootLinkRel.SEARCH_ALLE;
		const limit: number = 7;
		const action = VorgangActions.searchForPreview({ string: searchString });

		beforeEach(() => {
			vorgangFacade.getVorgangFilter.mockReturnValue(of(VorgangFilter.ALLE));
			apiRootFacade.getApiRoot.mockReturnValue(of(createStateResource(apiRoot)));
		})

		it('should call vorgang repository', () => {
			actions = of(action);

			effects.searchForPreview$.subscribe();

			expect(vorgangRepository.searchVorgaengeBy).toHaveBeenCalledWith(apiRoot, searchString, linkRel, limit);
		})

		it('should call apiRootFacade getApiRoot', () => {
			actions = of(action);

			effects.searchForPreview$.subscribe();

			expect(apiRootFacade.getApiRoot).toHaveBeenCalled();
		})

		it('should call vorgangFacade getVorgangFilter', () => {
			actions = of(action);

			effects.searchForPreview$.subscribe();

			expect(vorgangFacade.getVorgangFilter).toHaveBeenCalled();
		})

		it('should dispatch searchForPreviewSuccess action', () => {
			vorgangRepository.searchVorgaengeBy.mockReturnValue(of(vorgangList));

			actions = hot('-a-|', { a: action });

			const expected = hot('-a-|', { a: VorgangActions.searchForPreviewSuccess({ vorgangList }) });
			expect(effects.searchForPreview$).toBeObservable(expected);
		})

		it('should dispatch searchForPreviewFailure action', () => {
			const error: HttpErrorResponse = createHttpErrorResponse();
			const errorResponse = cold('-#', {}, error);
			vorgangRepository.searchVorgaengeBy = jest.fn(() => errorResponse);

			const expected = cold('--c', { c: VorgangActions.searchForPreviewFailure({ httpErrorResponse: error }) });
			actions = hot('-a', { a: action });

			expect(effects.searchForPreview$).toBeObservable(expected);
		})
	})

	describe('search error', () => {
		const action = VorgangActions.searchForPreviewFailure({ httpErrorResponse: null });
		const error: HttpErrorResponse = createHttpErrorResponse();

		it('should trigger showSearchError$', () => {
			actions = of(action);
			effects.showSearchError = jest.fn();

			effects.showSearchError$.subscribe();

			expect(effects.showSearchError).toHaveBeenCalled();
		})

		it('should call snackbarService.showError if HTTP status code is NOT 503 ', () => {
			error.error.status = HttpStatusCode.ExpectationFailed;

			effects.showSearchError(error);

			expect(snackbarService.showError).not.toHaveBeenCalled();
		})

		it('should call snackbarService.showError if HTTP status code is 503 ', () => {
			error.error.status = HttpStatusCode.ServiceUnavailable;

			effects.showSearchError(error);

			expect(snackbarService.showError).toHaveBeenCalled();
		})
	})

	//VorgangWithEingang
	describe('loadVorgangWithEingang', () => {

		const resourceUri: ResourceUri = faker.internet.url();
		const additionalSuccessAction = () => DUMMY_ACTION;
		const additionalFailureAction = (error: HttpErrorResponse) => DUMMY_ACTION;
		const additionalActions: AdditionalActions = { additionalSuccessAction, additionalFailureAction };

		const action: TypedAction<string> = VorgangActions.loadVorgangWithEingang({ resourceUri });

		describe('on success', () => {

			beforeEach(() => {
				vorgangRepository.getVorgang.mockReturnValue(of(vorgangWithEingang));
			})

			it('should dispatch action', () => {
				actions = hot('-a-|', { a: action });

				const expected = hot('-a-|', { a: VorgangActions.loadVorgangWithEingangSuccess({ vorgangWithEingang }) });
				expect(effects.loadVorgangWithEingang$).toBeObservable(expected);
			})

			it('should handle additional action', () => {
				const action: TypedAction<string> = VorgangActions.loadVorgangWithEingang({ resourceUri, additionalActions });

				actions = hot('-a', { a: action });

				const expected = cold('-(bc)', {
					b: VorgangActions.loadVorgangWithEingangSuccess({ vorgangWithEingang }),
					c: additionalSuccessAction()
				});
				expect(effects.loadVorgangWithEingang$).toBeObservable(expected);
			})
		})

		describe('on failure', () => {

			const apiError: ApiError = createApiError()
			const error = { error: { error: apiError } };

			beforeEach(() => {
				const errorResponse = cold('-#', {}, error);
				vorgangRepository.getVorgang = jest.fn(() => errorResponse);
			})

			it('should dispatch action', () => {
				actions = hot('-a', { a: action });

				const expected = cold('--c', { c: VorgangActions.loadVorgangWithEingangFailure({ apiError }) });
				expect(effects.loadVorgangWithEingang$).toBeObservable(expected);
			})

			it('should handle additional action', () => {
				const action: TypedAction<string> = VorgangActions.loadVorgangWithEingang({ resourceUri, additionalActions });

				actions = hot('-a', { a: action });

				const expected = cold('--(bc)', {
					b: VorgangActions.loadVorgangWithEingangFailure({ apiError }),
					c: additionalFailureAction(<HttpErrorResponse>error)
				});
				expect(effects.loadVorgangWithEingang$).toBeObservable(expected);
			})
		})
	})

	describe('loadVorgangWithEingangSuccess', () => {

		it('should dispatch "loadPendingCommandList" action if link exists', () => {
			const vorgangWithEingang: VorgangWithEingangResource = createVorgangWithEingangResource([VorgangWithEingangLinkRel.PENDING_COMMANDS]);
			const action = VorgangActions.loadVorgangWithEingangSuccess({ vorgangWithEingang });

			actions = hot('-a-|', { a: action });

			const expected = hot('-a-|', { a: VorgangActions.loadPendingCommandList({ vorgangWithEingang }) });
			expect(effects.loadVorgangWithEingangSuccess$).toBeObservable(expected);
		})

		it('should dispatch empty if link not exists', () => {
			const action = VorgangActions.loadVorgangWithEingangSuccess({ vorgangWithEingang });

			actions = hot('a', { a: action });

			expect(effects.loadVorgangWithEingangSuccess$).toBeObservable(cold('-'));
		})
	})

	describe('loadPendingCommandList', () => {

		const action = VorgangActions.loadPendingCommandList({ vorgangWithEingang });
		const props: LoadCommandListProps = <any>{};

		it('should dispatch "loadPendingCommandList" action', () => {
			effects.createLoadPendingCommandListProps = jest.fn().mockReturnValue(props);

			actions = hot('-a-|', { a: action });

			const expected = hot('-a-|', { a: loadCommandList(props) });
			expect(effects.loadPendingCommandList$).toBeObservable(expected);
		})

		it('should create props', () => {
			effects.createLoadPendingCommandListProps = jest.fn()
			actions = of(action);

			effects.loadPendingCommandList$.subscribe();

			expect(effects.createLoadPendingCommandListProps).toHaveBeenCalledWith(vorgangWithEingang);
		})

		describe('createLoadPendingCommandListProps', () => {

			it('should have uri', () => {
				const props: LoadCommandListProps = effects.createLoadPendingCommandListProps(vorgangWithEingang);

				expect(props.resource).toBe(vorgangWithEingang);
			})

			it('should have fileName', () => {
				const props: LoadCommandListProps = effects.createLoadPendingCommandListProps(vorgangWithEingang);

				expect(props.linkRel).toBe(VorgangWithEingangLinkRel.PENDING_COMMANDS);
			})

			it('should have as success action', () => {
				const props: LoadCommandListProps = effects.createLoadPendingCommandListProps(vorgangWithEingang);

				const commandList: CommandListResource = createCommandListResource();
				expect(props.successAction(commandList).type).toEqual('[Vorgang/API] Load pending command list Success');
			})

			it('should have as failure action', () => {
				const props: LoadCommandListProps = effects.createLoadPendingCommandListProps(vorgangWithEingang);

				const apiError: ApiError = createApiError();
				expect(props.failureAction(apiError).type).toEqual('[Vorgang/API] Load pending command list Failure');
			})
		})
	})

	describe('loadRepresentationList', () => {

		const action = VorgangActions.loadRepresentationList({ vorgangWithEingang });
		const props: LoadBinaryFileListProps = <any>{};

		it('should dispatch "loadRepresentationList" action', () => {
			effects.createLoadRepresentationListProps = jest.fn().mockReturnValue(props);

			actions = hot('-a-|', { a: action });

			const expected = hot('-a-|', { a: loadBinaryFileList(props) });
			expect(effects.loadRepresentationList$).toBeObservable(expected);
		})

		it('should create props', () => {
			effects.createLoadRepresentationListProps = jest.fn()
			actions = of(action);

			effects.loadRepresentationList$.subscribe();

			expect(effects.createLoadRepresentationListProps).toHaveBeenCalledWith(vorgangWithEingang);
		})

		describe('createLoadRepresentationListProps', () => {

			it('should have resource', () => {
				const props: LoadBinaryFileListProps = effects.createLoadRepresentationListProps(vorgangWithEingang);

				expect(props.resource).toBe(vorgangWithEingang);
			})

			it('should have linkRel', () => {
				const props: LoadBinaryFileListProps = effects.createLoadRepresentationListProps(vorgangWithEingang);

				expect(props.linkRel).toBe(VorgangWithEingangLinkRel.REPRESENTATIONS);
			})

			it('should have success action', () => {
				const props: LoadBinaryFileListProps = effects.createLoadRepresentationListProps(vorgangWithEingang);

				const binaryFileList: BinaryFileListResource = createBinaryFileListResource();
				expect(props.successAction(binaryFileList).type).toEqual('[Vorgang] Load RepresentationList Success');
			})

			it('should have failure action', () => {
				const props: LoadBinaryFileListProps = effects.createLoadRepresentationListProps(vorgangWithEingang);

				const apiError: ApiError = createApiError();
				expect(props.failureAction(apiError).type).toEqual('[Vorgang] Load RepresentationList Failure');
			})
		})
	})

	describe('loadAttachmentList', () => {

		const action = VorgangActions.loadAttachmentList({ vorgangWithEingang });
		const props: LoadBinaryFileListProps = <any>{};

		it('should dispatch "loadAttachmentList" action', () => {
			effects.createLoadAttachmentListProps = jest.fn().mockReturnValue(props);

			actions = hot('-a-|', { a: action });

			const expected = hot('-a-|', { a: loadBinaryFileList(props) });
			expect(effects.loadAttachmentList$).toBeObservable(expected);
		})

		it('should create props', () => {
			effects.createLoadAttachmentListProps = jest.fn()
			actions = of(action);

			effects.loadAttachmentList$.subscribe();

			expect(effects.createLoadAttachmentListProps).toHaveBeenCalledWith(vorgangWithEingang);
		})

		describe('createLoadAttachmentListProps', () => {

			it('should have resource', () => {
				const props: LoadBinaryFileListProps = effects.createLoadAttachmentListProps(vorgangWithEingang);

				expect(props.resource).toBe(vorgangWithEingang);
			})

			it('should have linkRel', () => {
				const props: LoadBinaryFileListProps = effects.createLoadAttachmentListProps(vorgangWithEingang);

				expect(props.linkRel).toBe(VorgangWithEingangLinkRel.ATTACHMENTS);
			})

			it('should have success action', () => {
				const props: LoadBinaryFileListProps = effects.createLoadAttachmentListProps(vorgangWithEingang);

				const binaryFileList: BinaryFileListResource = createBinaryFileListResource();
				expect(props.successAction(binaryFileList).type).toEqual('[Vorgang] Load AttachmentList Success');
			})

			it('should have failure action', () => {
				const props: LoadBinaryFileListProps = effects.createLoadAttachmentListProps(vorgangWithEingang);

				const apiError: ApiError = createApiError();
				expect(props.failureAction(apiError).type).toEqual('[Vorgang] Load AttachmentList Failure');
			})
		})
	})

	describe('exportVorgang', () => {

		const blob: Blob = new Blob();
		const fileName: string = faker.random.alphaNumeric();
		const blobWithFileName: BlobWithFileName = { blob, fileName };

		const action = VorgangActions.exportVorgang({ vorgangWithEingang });

		it('should call repository', () => {
			actions = hot('-a', { a: action });

			effects.exportVorgang$.subscribe(() => {
				expect(vorgangRepository.export).toHaveBeenCalledWith(vorgangWithEingang);
			});
		});

		it('should dispatch success action on no error', () => {
			vorgangRepository.export.mockReturnValue(of(blobWithFileName));

			actions = hot('-a', { a: action });

			const expected: ColdObservable = cold('-(bc)', {
				b: BinaryFileActions.saveFile({ fileData: blob, fileName }),
				c: VorgangActions.exportVorgangSuccess()
			});
			expect(effects.exportVorgang$).toBeObservable(expected);
		});

		it('should dispatch failure action', () => {
			const apiError: ApiError = createApiError()
			const error = { error: { error: apiError } };
			const errorResponse = cold('-#', {}, error);
			vorgangRepository.export = jest.fn(() => errorResponse);

			const expected = cold('--c', { c: VorgangActions.exportVorgangFailure({ apiError }) });
			actions = hot('-a', { a: action });

			expect(effects.exportVorgang$).toBeObservable(expected);
		})
	});
});