/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.net.URLDecoder;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Objects;
import java.util.Set;

import org.apache.commons.lang3.StringUtils;

import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.databind.BeanProperty;
import com.fasterxml.jackson.databind.DeserializationContext;
import com.fasterxml.jackson.databind.JavaType;
import com.fasterxml.jackson.databind.deser.ContextualDeserializer;
import com.fasterxml.jackson.databind.deser.std.StdDeserializer;

import de.itvsh.kop.common.datatype.StringBasedValue;
import lombok.Getter;

abstract class AbstractLinkedResourceDeserializer extends StdDeserializer<Object> implements ContextualDeserializer {

	private static final long serialVersionUID = 1L;

	@Getter
	private BeanProperty beanProperty;

	@Getter
	private final JavaType targetType;

	protected AbstractLinkedResourceDeserializer() {
		super(Object.class);
		targetType = null;
	}

	protected AbstractLinkedResourceDeserializer(BeanProperty beanProperty) {
		super(Object.class);
		this.beanProperty = beanProperty;
		this.targetType = beanProperty.getType();
	}

	@Override
	public Object deserialize(JsonParser jsonParser, DeserializationContext ctxt) throws IOException {
		if (jsonParser.isExpectedStartArrayToken()) {
			Collection<Object> idList = targetType.getRawClass().isAssignableFrom(Set.class) ? new HashSet<>() : new ArrayList<>();

			while (!jsonParser.nextToken().isStructEnd()) {
				idList.add(extractId(jsonParser.getText()));
			}
			return idList;
		} else {
			return extractId(jsonParser.getText());
		}
	}

	Object extractId(String url) {
		Class<?> type;
		if (targetType.isCollectionLikeType()) {
			type = targetType.getContentType().getRawClass();
		} else {
			type = targetType.getRawClass();
		}

		if (String.class.isAssignableFrom(type)) {
			return extractStringId(url);
		}
		if (Long.class.isAssignableFrom(type) || Long.TYPE.isAssignableFrom(type)) {
			return extractLongId(url);
		}
		if (StringBasedValue.class.isAssignableFrom(type)) {
			return extractStringBasedValue(type, url);
		}
		return buildByBuilder(url);
	}

	abstract Object buildByBuilder(String url);

	public static Long extractLongId(String uri) {
		var trimedUri = StringUtils.trimToNull(uri);
		if (Objects.isNull(trimedUri)) {
			return null;
		}
		return Long.parseLong(URLDecoder.decode(trimedUri.substring(trimedUri.lastIndexOf('/') + 1), StandardCharsets.UTF_8));
	}

	private StringBasedValue extractStringBasedValue(Class<?> type, String url) {
		String value = extractStringId(url);
		Method fromMethod;
		try {
			fromMethod = type.getMethod("from", String.class);
		} catch (NoSuchMethodException e) {
			throw new IllegalStateException(
					String.format("Cannot generate Id from type '%s'. Missing 'from' Method.", targetType.getRawClass().getSimpleName()));
		}
		try {
			return (StringBasedValue) fromMethod.invoke(null, value);
		} catch (IllegalAccessException | IllegalArgumentException | InvocationTargetException e) {
			throw new IllegalStateException(
					String.format("Cannot generate Id from type '%s'. Error calling 'from' Method.", targetType.getRawClass().getSimpleName()),
					e);
		}
	}

	public static String extractStringId(String url) {
		return URLDecoder.decode(url.substring(url.lastIndexOf('/') + 1), StandardCharsets.UTF_8);
	}
}
