/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.vorgang.command;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.*;

import java.util.Objects;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.hateoas.EntityModel;
import org.springframework.hateoas.LinkRelation;
import org.springframework.hateoas.server.RepresentationModelProcessor;
import org.springframework.stereotype.Component;

import de.ozgcloud.alfa.common.ModelBuilder;
import de.ozgcloud.alfa.common.command.CommandController;
import de.ozgcloud.alfa.common.command.CommandController.CommandByRelationController;
import de.ozgcloud.alfa.common.user.CurrentUserService;
import de.ozgcloud.alfa.common.user.UserRole;
import de.ozgcloud.alfa.vorgang.Vorgang.VorgangStatus;
import de.ozgcloud.alfa.vorgang.VorgangWithEingang;
import de.ozgcloud.alfa.vorgang.forwarding.ForwardingController;

@Component
public class VorgangWithEingangCommandProzessor implements RepresentationModelProcessor<EntityModel<VorgangWithEingang>> {

	static final LinkRelation REL_VORGANG_FORWARD = LinkRelation.of("forward");
	static final LinkRelation REL_PENDING_COMMANDS = LinkRelation.of("pending-commands");

	@Autowired
	private CurrentUserService userService;
	@Autowired
	private ForwardingController forwardingController;
	@Autowired
	private CommandController commandController;

	@Override
	public EntityModel<VorgangWithEingang> process(EntityModel<VorgangWithEingang> model) {
		var vorgang = model.getContent();
		if (Objects.isNull(vorgang)) {
			return model;
		}
		return ModelBuilder.fromModel(model)
				.ifMatch(this::isForwardingAllowed)
				.addLink(linkTo(
						methodOn(CommandByRelationController.class).createCommand(vorgang.getId(), vorgang.getId(), vorgang.getVersion(),
								null)).withRel(REL_VORGANG_FORWARD))
				.ifMatch(this::existsPendingCommands)
				.addLink(linkTo(methodOn(CommandController.class).getPendingCommands(true, vorgang.getId())).withRel(REL_PENDING_COMMANDS))
				.buildModel();
	}

	boolean isForwardingAllowed(VorgangWithEingang vorgang) {
		return userService.hasRole(UserRole.EINHEITLICHER_ANSPRECHPARTNER)
				&& (vorgang.getStatus() == VorgangStatus.NEU
				|| (vorgang.getStatus() == VorgangStatus.IN_BEARBEITUNG
				&& hasForwardingFailed(vorgang)));
	}

	private boolean hasForwardingFailed(VorgangWithEingang vorgang) {
		return !forwardingController.findFailedForwardings(vorgang.getId()).isEmpty();
	}

	private boolean existsPendingCommands(VorgangWithEingang vorgang) {
		return commandController.existsPendingCommands(vorgang.getId());
	}
}