/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.vorgang.forwarding;

import java.io.IOException;
import java.io.InputStream;
import java.nio.charset.StandardCharsets;
import java.util.Collections;
import java.util.HashSet;
import java.util.Optional;
import java.util.Scanner;
import java.util.Set;
import java.util.concurrent.CompletableFuture;
import java.util.regex.Pattern;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import de.itvsh.kop.common.errorhandling.TechnicalException;
import lombok.extern.log4j.Log4j2;

@Log4j2
@Service
@ConditionalOnProperty(prefix = LandesnetzInfoProperties.LNINFO_CONFIG_PREFIX, name = { "url" })
class LandesnetzInfoReadService {

	static final Pattern SH_SECTION_START = Pattern.compile(".*Beginn Schleswig-Holstein.*", Pattern.CASE_INSENSITIVE);
	static final Pattern SH_SECTION_END = Pattern.compile(".*Ende Schleswig-Holstein.*", Pattern.CASE_INSENSITIVE);

	@Autowired
	private LandesnetzInfoProperties properties;

	@Async
	public CompletableFuture<Set<String>> readLandesnetzInfo() {
		try {
			return getInputStream()
					.map(this::readFile)
					.map(CompletableFuture::completedFuture)
					.orElseGet(() -> {
						LOG.warn("No Landesnetz-Information found");
						return CompletableFuture.completedFuture(Collections.emptySet());
					});
		} catch (RuntimeException e) {
			LOG.error("Error on reading Landesnetz-Infos.", e);
		}
		return CompletableFuture.completedFuture(Collections.emptySet());
	}

	private Optional<InputStream> getInputStream() {
		try {
			return Optional.ofNullable(properties.getUrl().getInputStream());
		} catch (IOException e) {
			throw new TechnicalException("Error on reading Landesnetz-Infos", e);
		}
	}

	Set<String> readFile(InputStream fileInputStream) {
		var fileScanner = new Scanner(fileInputStream, StandardCharsets.UTF_8);
		while (fileScanner.hasNextLine()) {
			String line = fileScanner.nextLine();
			if (isStartOfSection(line)) {
				return readSection(fileScanner);
			}
		}
		return Collections.emptySet();
	}

	boolean isStartOfSection(String line) {
		return SH_SECTION_START.matcher(line).matches();
	}

	Set<String> readSection(Scanner sectionScanner) {
		Set<String> result = new HashSet<>();
		while (sectionScanner.hasNextLine()) {
			String line = sectionScanner.nextLine();
			if (isEndOfSection(line)) {
				return Collections.unmodifiableSet(result);
			}
			if (isCommentedLine(line)) {
				continue;
			}
			getFirstColumn(line).ifPresent(result::add);
		}

		return Collections.emptySet();
	}

	boolean isEndOfSection(String line) {
		return SH_SECTION_END.matcher(line).matches();
	}

	boolean isCommentedLine(String line) {
		return line.startsWith("#");
	}

	Optional<String> getFirstColumn(String line) {
		return Optional.of(line.split("\\s+")[0]).filter(StringUtils::isNoneBlank);
	}

}
