/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa;

import static org.assertj.core.api.Assertions.*;
import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import com.auth0.jwt.exceptions.JWTVerificationException;

import de.ozgcloud.alfa.common.binaryfile.FileId;
import de.ozgcloud.alfa.common.downloadtoken.DownloadTokenProperties;
import de.ozgcloud.alfa.common.user.UserProfileTestFactory;
import de.ozgcloud.alfa.vorgang.ZustaendigeStelleTestFactory;
import io.jsonwebtoken.Claims;
import io.jsonwebtoken.Jwts;
import io.jsonwebtoken.SignatureAlgorithm;

class JwtTokenUtilTest {

	@Spy
	@InjectMocks
	private JwtTokenUtil jwtTokenUtil;
	@Mock
	private DownloadTokenProperties downloadTokenProperties;

	private static final String TOKEN_SECRET = "t0pS3cr3t".repeat(8);
	private static final int TOKEN_VALIDITY = 60000;

	@BeforeEach
	public void initTest() throws Exception {
		when(downloadTokenProperties.getSecret()).thenReturn(TOKEN_SECRET);
	}

	@Nested
	@DisplayName("Verify token generation")
	class TestGenerateToken {

		private String generatedToken;

		@BeforeEach
		void initTest() {
			when(downloadTokenProperties.getSecret()).thenReturn(TOKEN_SECRET);
			when(downloadTokenProperties.getValidity()).thenReturn(TOKEN_VALIDITY);

			generatedToken = jwtTokenUtil.generateToken(FileId.createNew(), UserProfileTestFactory.create());
		}

		@Test
		void userId() {
			var userId = getParsedBody().getSubject();

			assertThat(userId).isEqualTo(SecurityTestFactory.SUBJECT.toString());
		}

		@Test
		void expirationDate() {
			var before = new Date();
			var expirationDate = getParsedBody().getExpiration();
			var after = new Date(System.currentTimeMillis() + 900000);

			assertThat(expirationDate).isAfter(before).isBefore(after);
		}

		@Test
		void organisationseinheitIds() {
			var organisationseinheitIds = jwtTokenUtil.getOrganisationseinheitIdsFromToken(generatedToken);

			assertThat(organisationseinheitIds).isEqualTo(List.of(ZustaendigeStelleTestFactory.ORGANISATIONSEINHEITEN_ID));
		}

		private Claims getParsedBody() {
			return Jwts.parser().setSigningKey(TOKEN_SECRET.getBytes()).parseClaimsJws(generatedToken).getBody();
		}
	}

	@Nested
	class TestVerifyToken {

		@Test
		void shouldDoNotThrowExcepionOnValidToken() {
			var token = buildToken(UUID.randomUUID().toString(), TOKEN_SECRET, TOKEN_VALIDITY);

			jwtTokenUtil.verifyToken(token);
		}

		@Test
		void shouldThrowExceptionOnInvalidToken() {
			var token = buildToken(UUID.randomUUID().toString(), "invalid_token".repeat(5), TOKEN_VALIDITY);

			assertThrows(JWTVerificationException.class, () -> jwtTokenUtil.verifyToken(token));
		}

		@Test
		void shouldThrowExceptionOnTimeExpired() {
			var token = buildToken(UUID.randomUUID().toString(), TOKEN_SECRET, -1000);

			assertThrows(JWTVerificationException.class, () -> jwtTokenUtil.verifyToken(token));
		}

		private String buildToken(String subject, String token, int expiredTime) {
			Map<String, Object> claims = new HashMap<>();
			claims.put(JwtTokenUtil.FIRSTNAME_CLAIM, SecurityTestFactory.USER_FIRSTNAME);
			claims.put(JwtTokenUtil.LASTNAME_CLAIM, SecurityTestFactory.USER_LASTNAME);
			claims.put(JwtTokenUtil.ROLE_CLAIM, SecurityTestFactory.AUTHORITIES);
			claims.put(JwtTokenUtil.FILEID_CLAIM, FileId.createNew());
			claims.put(JwtTokenUtil.ORGANSIATIONSEINHEIT_IDS_CLAIM, SecurityTestFactory.ORGANSIATIONSEINHEIT_IDS);

			return Jwts.builder()
					.setClaims(claims)
					.setSubject(subject)
					.setHeaderParam(JwtTokenUtil.TOKEN_TYP_KEY, JwtTokenUtil.TOKEN_TYPE)
					.setIssuer(JwtTokenUtil.TOKEN_ISSUER).setIssuedAt(new Date(System.currentTimeMillis()))
					.setExpiration(new Date(System.currentTimeMillis() + expiredTime))
					.setAudience(JwtTokenUtil.TOKEN_AUDIENCE)
					.signWith(SignatureAlgorithm.HS512, token.getBytes())
					.compact();
		}
	}
}