/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common.binaryfile;

import static org.assertj.core.api.Assertions.*;
import static org.hamcrest.CoreMatchers.*;
import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.request;

import java.io.IOException;
import java.io.OutputStream;
import java.util.List;
import java.util.concurrent.CompletableFuture;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.hateoas.CollectionModel;
import org.springframework.hateoas.EntityModel;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.MvcResult;
import org.springframework.test.web.servlet.ResultActions;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.method.annotation.StreamingResponseBody;

import de.itvsh.kop.common.errorhandling.TechnicalException;
import de.ozgcloud.alfa.common.file.OzgFile;
import de.ozgcloud.alfa.common.file.OzgFileTestFactory;
import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;

class BinaryFileControllerTest {

	private final String PATH = BinaryFileController.PATH;

	@Spy
	@InjectMocks
	private BinaryFileController controller;
	@Mock
	private BinaryFileService service;
	@Mock
	private BinaryFileModelAssembler modelAssembler;
	@Mock
	private FileIdMapper fileIdMapper;

	private MockMvc mockMvc;

	@BeforeEach
	void initTest() {
		mockMvc = MockMvcBuilders.standaloneSetup(controller).build();
	}

	@Nested
	class TestUploadFile {

		private CompletableFuture<FileId> fileIdFuture = CompletableFuture.completedFuture(BinaryFileTestFactory.FILE_ID);

		@BeforeEach
		void mockServiec() {
			when(service.uploadFile(any())).thenReturn(fileIdFuture);
		}

		@Test
		void shouldCallService() throws Exception {
			var uploadBinaryFile = UploadBinaryFileTestFactory.create();
			doReturn(uploadBinaryFile).when(controller).buildBinaryFileUploadRequest(any(), any(), any());

			callEndpoint();

			verify(service).uploadFile(uploadBinaryFile);
		}

		@Test
		void shouldReturnCreatedStatus() throws Exception {
			var result = callEndpoint();

			mockMvc.perform(asyncDispatch(result)).andExpect(status().isCreated());
		}

		@Test
		void shouldReturnLocation() throws Exception {
			var result = callEndpoint();

			mockMvc.perform(asyncDispatch(result))
					.andExpect(header().string("Location", containsString("/api/binaryFiles/" + BinaryFileTestFactory.FILE_ID)));
		}

		private MvcResult callEndpoint() throws Exception {
			return mockMvc.perform(multipart(PATH + "/" + VorgangHeaderTestFactory.ID + "/" + BinaryFileTestFactory.FIELD + "/file")
					.file(BinaryFileTestFactory.TEST_FILE))
					.andExpect(request().asyncStarted())
					.andReturn();
		}
	}

	@Nested
	class TestBuildBinaryFileUploadRequest {

		@Test
		void shouldContainsVorgangId() {
			var result = buildBinaryFileUploadRequest();

			assertThat(result.getVorgangId()).isEqualTo(VorgangHeaderTestFactory.ID);
		}

		@Test
		void shouldContainsField() {
			var result = buildBinaryFileUploadRequest();

			assertThat(result.getField()).isEqualTo(BinaryFileTestFactory.FIELD);
		}

		@Test
		void shouldContainsSize() {
			var result = buildBinaryFileUploadRequest();

			assertThat(result.getSize()).isEqualTo(OzgFileTestFactory.SIZE);
		}

		@Test
		void shouldContainsName() {
			var result = buildBinaryFileUploadRequest();

			assertThat(result.getName()).isEqualTo(OzgFileTestFactory.NAME);
		}

		@Test
		void shouldContainsContentType() {
			var result = buildBinaryFileUploadRequest();

			assertThat(result.getContentType()).isEqualTo(OzgFileTestFactory.CONTENT_TYPE);
		}

		@Test
		void shouldGetInputStreamFromFile() {
			buildBinaryFileUploadRequest();

			verify(controller).getInputStream(BinaryFileTestFactory.TEST_FILE);
		}

		private UploadBinaryFileRequest buildBinaryFileUploadRequest() {
			return controller.buildBinaryFileUploadRequest(VorgangHeaderTestFactory.ID, BinaryFileTestFactory.FIELD, BinaryFileTestFactory.TEST_FILE);
		}
	}

	@Nested
	class TestGetInputStreamOfFile {

		@Mock
		private MultipartFile file;

		@Test
		void shouldReturnStream() throws IOException {
			var uploadStream = file.getInputStream();

			when(file.getInputStream()).thenReturn(uploadStream);

			assertThat(controller.getInputStream(file)).isEqualTo(uploadStream);
		}

		@Test
		void shouldThrowException() throws IOException {
			when(file.getInputStream()).thenThrow(IOException.class);

			assertThrows(TechnicalException.class, () -> controller.getInputStream(file));
		}
	}

	@Nested
	class TestGetFileData {

		@Mock
		private StreamingResponseBody body;
		private OzgFile ozgFile = OzgFileTestFactory.create();

		@BeforeEach
		void mockService() {
			when(service.getFile(any())).thenReturn(ozgFile);
		}

		@Test
		void shouldCallServiceGetFile() throws Exception {
			callEndpoint();

			verify(service).getFile(OzgFileTestFactory.ID);
		}

		@Test
		void shouldBuildResponseEntity() throws Exception {
			doReturn(body).when(controller).createDownloadStreamingBody(OzgFileTestFactory.ID);

			callEndpoint();

			verify(controller).buildResponseEntity(ozgFile, body);
		}

		private ResultActions callEndpoint() throws Exception {
			return mockMvc.perform(get(PATH + "/" + OzgFileTestFactory.ID).contentType(MediaType.APPLICATION_OCTET_STREAM_VALUE))
					.andExpect(status().isOk());
		}
	}

	@Nested
	class TestCreateDownloadStreamingBody {

		@Mock
		private OutputStream out;

		@Test
		void shouldCallServiceWriteFileContent() throws IOException {
			controller.createDownloadStreamingBody(OzgFileTestFactory.ID).writeTo(out);

			verify(service).writeFileContent(any(), any());
		}
	}

	@Nested
	class TestGetFile {

		private OzgFile ozgFile = OzgFileTestFactory.create();

		@BeforeEach
		void mockServiec() {
			when(service.getFile(any())).thenReturn(ozgFile);
		}

		@Test
		void shouldCallService() throws Exception {
			callEndpoint();

			verify(service).getFile(BinaryFileTestFactory.FILE_ID);
		}

		@Test
		void shouldCallAssembler() throws Exception {
			callEndpoint();

			verify(modelAssembler).toModel(ozgFile);
		}

		private ResultActions callEndpoint() throws Exception {
			return mockMvc.perform(get(BinaryFileController.PATH + '/' + BinaryFileTestFactory.FILE_ID)
					.contentType(MediaType.APPLICATION_JSON_VALUE))
					.andExpect(status().isOk());
		}
	}

	@Nested
	class TestGetFiles {

		private OzgFile file = OzgFileTestFactory.create();

		@BeforeEach
		void mockServiec() {
			when(service.getFiles(any())).thenReturn(Stream.of(file));
			when(fileIdMapper.toString(any())).then(fileId -> fileId.toString());
		}

		@Test
		void shouldCallService() throws Exception {
			callEndpoint();

			verify(service).getFiles(any());
		}

		@Test
		void shouldCallAssembler() throws Exception {
			callEndpoint();

			verify(modelAssembler).toCollectionModel(Stream.of(file).collect(Collectors.toList()));
		}

		private CollectionModel<EntityModel<OzgFile>> callEndpoint() throws Exception {
			return controller.getFiles(List.of(BinaryFileTestFactory.FILE_ID));
		}
	}
}