/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common.user;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Optional;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.security.oauth2.jwt.Jwt;

import de.ozgcloud.alfa.JwtTestFactory;

class CurrentUserServiceTest {

	@Spy
	private CurrentUserService service;

	@Nested
	class TestGetOrganisationseinheit {

		@Mock
		private Jwt jwt;

		@Test
		void shouldReturnOrganisationseinheitIdsFromMap() {
			when(jwt.getClaim(CurrentUserService.USER_ATTRIBUTE_ORGANISATIONSEINHEIT_ID)).thenReturn(List.of("1", "2"));

			var result = service.getOrganisationsEinheitIds(jwt);

			assertThat(result).containsExactly("1", "2");
		}

		@Test
		void shouldReturnOrgaIdAsString() {
			when(jwt.getClaim(CurrentUserService.USER_ATTRIBUTE_ORGANISATIONSEINHEIT_ID)).thenReturn(List.of("1", 2));

			var result = service.getOrganisationsEinheitIds(jwt);

			assertThat(result).containsExactly("1", "2");
		}

		@Test
		void shouldReturnEmptyList() {
			when(jwt.getClaim(CurrentUserService.USER_ATTRIBUTE_ORGANISATIONSEINHEIT_ID)).thenReturn(Collections.emptyList());

			var result = service.getOrganisationsEinheitIds(jwt);

			assertThat(result).isEmpty();
		}

		@Test
		void shouldReturnEmptyListIfNotExists() {
			var result = service.getOrganisationsEinheitIds(null);

			assertThat(result).isEmpty();
		}
	}

	@Nested
	class TestGetUserId {

		@Test
		void shouldReturnValueFromClaim() {
			doReturn(Optional.of(UserProfileTestFactory.ID.toString())).when(service).getSingleClaimValue(any());

			var id = service.getUserId();

			assertThat(id).isEqualTo(UserProfileTestFactory.ID);
		}
	}

	@DisplayName("Get single claim value")
	@Nested
	class TestGetSingleClaimValue {

		@DisplayName("on existing claim")
		@Nested
		class TestOnExistingClaim {

			private static final String ATTRIBUTE_NAME = "userId";
			private static final String ATTRIBUTE_VALUE = "deadbeef";
			private static final Map<String, Object> CLAIMS = Map.of(ATTRIBUTE_NAME, ATTRIBUTE_VALUE);

			@Test
			void shouldReturnUserAttribute() {
				doReturn(Optional.of(JwtTestFactory.create(CLAIMS))).when(service).getCurrentSecurityToken();

				var attribute = service.getSingleClaimValue(ATTRIBUTE_NAME);

				assertThat(attribute).isPresent().hasValue(ATTRIBUTE_VALUE);
			}
		}

		@DisplayName("on missing claim")
		@Nested
		class TestOnMissingClaim {

			private static final String ATTRIBUTE_NAME = "non_existing_claim";
			private static final Map<String, Object> CLAIMS = Collections.<String, Object>singletonMap("dummyClaim", "DummyClaimValue");

			@Test
			void shouldReturnEmpty() {
				doReturn(Optional.of(JwtTestFactory.create(CLAIMS))).when(service).getCurrentSecurityToken();

				var attribute = service.getSingleClaimValue(ATTRIBUTE_NAME);

				assertThat(attribute).isEmpty();
			}
		}
	}
}