package de.ozgcloud.alfa.loeschanforderung;

import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;
import static org.springframework.security.test.web.servlet.request.SecurityMockMvcRequestPostProcessors.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import java.nio.charset.StandardCharsets;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.ResultActions;

import de.itvsh.kop.common.test.TestUtils;
import de.ozgcloud.alfa.common.attacheditem.VorgangAttachedItemService;
import de.ozgcloud.alfa.common.attacheditem.VorgangAttachedItemTestFactory;
import de.ozgcloud.alfa.common.command.CommandOrder;
import de.ozgcloud.alfa.common.command.CommandTestFactory;
import de.ozgcloud.alfa.common.user.UserRole;
import de.ozgcloud.alfa.vorgang.VorgangController;
import de.ozgcloud.alfa.vorgang.VorgangWithEingangTestFactory;

@WithMockUser
@AutoConfigureMockMvc
@SpringBootTest
class LoeschAnforderungCommandControllerITCase {

	@Autowired
	private MockMvc mockMvc;

	@MockBean
	private LoeschAnforderungService loeschAnforderungService;

	@MockBean
	private VorgangAttachedItemService vorgangAttachedItemService;

	@MockBean
	private VorgangController vorgangController;

	@Nested
	class TestCreateCommand {

		private final LoeschAnforderung loeschAnforderung = LoeschAnforderungTestFactory.create();

		@DisplayName("Create VORGANG_LOESCHEN command")
		@Nested
		class TestVorgangLoeschen {

			@BeforeEach
			void init() {
				when(loeschAnforderungService.vorgangLoeschen(any(), anyString())).thenReturn(CommandTestFactory.create());
				when(loeschAnforderungService.getById(LoeschAnforderungTestFactory.ID)).thenReturn(loeschAnforderung);
			}

			@Test
			void shouldReturnForbidden() throws Exception {
				when(loeschAnforderungService.isAllowedToVorgangLoeschen(loeschAnforderung)).thenReturn(false);

				var response = doRequest(CommandOrder.VORGANG_LOESCHEN);

				response.andExpect(status().isForbidden());
			}

			@WithMockUser(roles = UserRole.VERWALTUNG_LOESCHEN)
			@Test
			void shouldReturnCreated() throws Exception {
				when(loeschAnforderungService.isAllowedToVorgangLoeschen(loeschAnforderung)).thenReturn(true);

				var response = doRequest(CommandOrder.VORGANG_LOESCHEN).andExpect(status().isCreated());

				response.andExpect(status().isCreated());
			}

		}

		@DisplayName("Create LOESCH_ANFORDERUNG_ZURUECKNEHMEN command")
		@Nested
		class TestDeleteLoeschAnforderung {

			@BeforeEach
			void init() {
				var vorgangAttachedItem = VorgangAttachedItemTestFactory.create();
				var vorgang = VorgangWithEingangTestFactory.create();

				when(vorgangAttachedItemService.getById(LoeschAnforderungTestFactory.ID)).thenReturn(vorgangAttachedItem);
				when(vorgangController.getVorgang(LoeschAnforderungTestFactory.VORGANG_ID)).thenReturn(vorgang);
				when(loeschAnforderungService.zuruecknehmen(vorgangAttachedItem, vorgang)).thenReturn(CommandTestFactory.create());
			}

			@WithMockUser(roles = UserRole.VERWALTUNG_USER)
			@Test
			void shouldReturnCreated() throws Exception {
				when(loeschAnforderungService.isAllowedToVorgangLoeschen(loeschAnforderung)).thenReturn(false);

				var response = doRequest(CommandOrder.LOESCH_ANFORDERUNG_ZURUECKNEHMEN);

				response.andExpect(status().isCreated());
			}

			@WithMockUser(roles = UserRole.VERWALTUNG_POSTSTELLE)
			@Test
			void shouldReturnForbidden() throws Exception {
				when(loeschAnforderungService.isAllowedToVorgangLoeschen(loeschAnforderung)).thenReturn(false);

				var response = doRequest(CommandOrder.LOESCH_ANFORDERUNG_ZURUECKNEHMEN);

				response.andExpect(status().isForbidden());
			}
		}
	}

	private ResultActions doRequest(CommandOrder commandOrder) throws Exception {
		var requestBody = TestUtils.loadTextFile(
				"jsonTemplates/command/createVorgangCommand.json.tmpl",
				commandOrder.name());

		return mockMvc.perform(post(LoeschAnforderungCommandController.BASE_PATH, LoeschAnforderungTestFactory.ID)
				.with(csrf())
				.content(requestBody).contentType(MediaType.APPLICATION_JSON).characterEncoding(StandardCharsets.UTF_8.name()));
	}
}